package org.frogpond.metadata;

import org.frogpond.MetadataException;
import org.frogpond.annotation.LilyField;
import org.frogpond.annotation.Versioned;
import org.frogpond.model.Primitive;
import org.frogpond.model.PrimitiveValueTypePlaceholder;
import org.frogpond.model.PropertyEntry;
import org.frogpond.utils.MetadataUtilities;
import org.lilyproject.repository.api.FieldType;
import org.lilyproject.repository.api.RecordType;
import org.lilyproject.repository.api.Scope;
import org.lilyproject.repository.api.ValueType;
import org.lilyproject.repository.impl.FieldTypeImpl;
import org.lilyproject.repository.impl.RecordTypeImpl;
import org.lilyproject.repository.impl.ValueTypeImpl;

import java.util.Collection;

public class LilyTypeHelper {
    public static RecordType createRecordType(Class<?> objectType) {
        return new RecordTypeImpl(null, MetadataUtilities.getQualifiedName(objectType));
    }

    public static FieldType getFieldType(PropertyEntry property) throws MetadataException {
        // -- get the annotation
        LilyField fieldAnnotation = property.getAnnotation(LilyField.class);

        if (fieldAnnotation == null)
            throw new MetadataException(String.format(
                    "The field with name %s does not contain a LilyField annotation.",
                    property.getName()
            ));

        Versioned versionedAnnotation = property.getAnnotation(Versioned.class);
        Scope scope = (versionedAnnotation == null) ? Scope.NON_VERSIONED : Scope.VERSIONED;

        return new FieldTypeImpl(
                null, getValueType(property, fieldAnnotation), property.getName(), scope
        );
    }

    protected static ValueType getValueType(PropertyEntry property, LilyField fieldAnnotation) throws MetadataException {
        Primitive primitive = fieldAnnotation.primitive();

        if (primitive == null || primitive == Primitive.None) {
            if (Collection.class.isAssignableFrom(property.getType())) {
                throw new MetadataException(String.format(
                        "The field with name %s is a collection in which case the annotation should contain the actual valueType.",
                        property.getName()
                ));

            } else {
                primitive = Primitive.primitiveOf(property.getType());
            }
        }

        return new ValueTypeImpl(
                new PrimitiveValueTypePlaceholder(primitive),
                (Collection.class.isAssignableFrom(property.getType())),
                false
        );
    }
}
