package org.frogpond.model;

import org.lilyproject.repository.api.PrimitiveValueType;

import java.io.File;
import java.net.URI;
import java.util.Date;

public enum Primitive {
    None("", "", null, ""),
    String("STRING", "string", String.class, "string"),
    Integer("INTEGER", "number", Integer.class, "int"),
    Long("LONG", "number", Long.class, "long"),
    Double("DOUBLE", "number", Double.class, "double"),
    Decimal("DECIMAL", "number", Float.class, "float"),
    Boolean("BOOLEAN", "boolean", Boolean.class, "string"),
    Date("DATE", "string", Date.class, "date"),
    DateTime("DATETIME", "string", Date.class, "date"),
    Uri("URI", "string", URI.class, "string"),
    Link("LINK", "string", Object.class, "string"),
    Blob("BLOB", "object", File.class, "text");

    private String lilyType;
    private String jsonType;
    private Class<?> javaType;
    private String solrType;

    private PrimitiveValueType cachedValueType;

    private Primitive(String lilyType, String jsonType, Class<?> javaType, String solrType) {
        this.lilyType = lilyType;
        this.jsonType = jsonType;
        this.javaType = javaType;
        this.solrType = solrType;
    }

    public String getLilyType() {
        return lilyType;
    }

    public String getJsonType() {
        return jsonType;
    }

    public Class<?> getJavaType() {
        return javaType;
    }

    public String getSolrType() {
        return solrType;
    }

    public static Primitive primitiveOf(Class<?> cls) {
        String clsName = cls.getName();

        for (Primitive primitive : Primitive.values())
            if ((primitive != None) &&
                    (clsName.equals(primitive.javaType.getName())))
                return primitive;

        return None;
    }
}
