package org.frogpond.metadata.extractor;

import org.frogpond.MetadataException;
import org.frogpond.demo.model.Author;
import org.frogpond.demo.model.Book;
import org.frogpond.demo.model.Publisher;
import org.frogpond.metadata.RecordTypeMetadata;
import org.frogpond.demo.model.Review;
import org.junit.Assert;
import org.junit.Test;
import org.lilyproject.repository.api.QName;

public class ReflectiveMetadataExtractorTest {
    private ReflectiveMetadataExtractor extractor = new ReflectiveMetadataExtractor();

    @Test
    public void testExtractWithUserDefinedRecordId() throws MetadataException {
        // -- extract the metadata from the Book mock
        MetadataContainer container = extractor.extract(Book.class);

        // -- validate the container class
        validateRecordType(container.getRecordTypeMetadata(), Book.class, 1, 5, true);
    }

    @Test
    public void testExtractWithGeneratedRecordId() throws MetadataException {
        // -- extract the metadata from the Book mock
        MetadataContainer container = extractor.extract(Publisher.class);

        // -- validate the container class
        validateRecordType(container.getRecordTypeMetadata(), Publisher.class, 0, 2, true);
    }

    @Test
    public void testExtractWithMetadataOnPropertiesAndFields() throws MetadataException {
        // -- extract the metadata from the Book mock
        MetadataContainer container = extractor.extract(Author.class);

        // -- validate the container class
        validateRecordType(container.getRecordTypeMetadata(), Author.class, 0, 3, true);
    }

    @Test
    public void testExtractWithDuplicateMetadataOnPropertiesAndFields() throws MetadataException {
        // -- extract the metadata from the Book mock
        MetadataContainer container = extractor.extract(Review.class);

        // -- validate the container class
        validateRecordType(container.getRecordTypeMetadata(), Review.class, 0, 4, true);
    }

    protected void validateRecordType(RecordTypeMetadata recordTypeMetadata, Class expectedType, int expectedVariantCount, int expectedNumberOfFields, boolean hasIndex) {
        Assert.assertEquals(expectedType, recordTypeMetadata.getJavaType());
        Assert.assertEquals(new QName("org.frogpond.demo.model", expectedType.getSimpleName()), recordTypeMetadata.getName());

        // -- index
        if (hasIndex) Assert.assertNotNull(recordTypeMetadata.getIndex());
        else Assert.assertNull(recordTypeMetadata.getIndex());

        // -- record type
        Assert.assertNotNull(recordTypeMetadata.getRecordType());
        Assert.assertNotNull(recordTypeMetadata.getRecordType().getVersion());

        // -- record id
        Assert.assertNotNull(recordTypeMetadata.getRecordIdProperty());

        // -- variants
        Assert.assertNotNull(recordTypeMetadata.getVariants());
        Assert.assertEquals(expectedVariantCount, recordTypeMetadata.getVariants().size());

        // -- fields
        Assert.assertNotNull(recordTypeMetadata.getFields());
        Assert.assertEquals(expectedNumberOfFields, recordTypeMetadata.getFields().size());
    }
}
