package org.frogpond.utils;

import org.frogpond.demo.model.Publisher;
import org.frogpond.metadata.extractor.ReflectiveMetadataExtractor;
import org.frogpond.metadata.store.DefaultMetadataStoreFactory;
import org.frogpond.metadata.store.MetadataStore;
import org.frogpond.demo.model.Author;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.lilyproject.client.LilyClient;
import org.lilyproject.repository.api.RecordId;
import org.lilyproject.repository.api.Repository;

public class LilyUtilitiesTest {
    public static final String ZOOKEEPER_URL = "localhost:2181";

    private static Repository repository;
    private static MetadataStore metadataStore;

    @BeforeClass
    public static void initialize() throws Exception {
        // -- create the lily client
        LilyClient lilyClient = new LilyClient(ZOOKEEPER_URL, 10000);
        repository = lilyClient.getRepository();

        // -- construct the metadata store
        DefaultMetadataStoreFactory metadataStoreFactory = new DefaultMetadataStoreFactory();
        metadataStoreFactory.setMetadataExtractor(new ReflectiveMetadataExtractor());
        metadataStoreFactory.setBasePackage("org.frogpond.demo.model");
        metadataStore = metadataStoreFactory.create();
    }

    @Test
    public void getUserDefinedRecordId() {
        // -- An author has a user defined recordId
        Author author = new Author("daan", "an excellent idiot", "daan.gerits@gmail.com");
        validateRecordId(LilyUtilities.getRecordId(
                repository, metadataStore.getRecordTypeMetadata(Author.class), author
        ), false);
    }

    @Test
    public void getGeneratedRecordId() {
        // -- A publisher has an auto-generated recordId
        Publisher publisher = new Publisher("My Publisher", "http://foo.bar");
        validateRecordId(LilyUtilities.getRecordId(
                repository, metadataStore.getRecordTypeMetadata(Publisher.class), publisher
        ), true);
    }

    protected void validateRecordId(RecordId result, boolean systemGenerated) {
        String prefix = (systemGenerated) ? "UUID" : "USER";

        Assert.assertTrue(result.toString().startsWith(prefix));
    }
}
